@@ -21,11 +21,6 @@ require Exporter ;
 our @EXPORT = qw(data) ;
 our @EXPORT_OK = @EXPORT ;
 
-use strict ;
-no warnings ;
-
-use XML::Smart::Entity qw(_add_basic_entity) ;
-
 ########
 # DATA #
 ########
@@ -42,14 +37,12 @@ sub data {
   {
     my $addroot ;
 
-    if ( $args{root} || ref $tree ne 'HASH' ) { $addroot = 1 ;}
+    if ( ref $tree ne 'HASH' ) { $addroot = 1 ;}
     else {
       my $ks = keys %$tree ;
       my $n = 1 ;
-      if (ref $$tree{'/nodes'} eq 'HASH')  { ++$n ;}
-      if (ref $$tree{'/order'} eq 'ARRAY') { ++$n ;}
-      #if (ref $$tree{'/nodes'} eq 'HASH')  { ++$n if (keys %{$$tree{'/nodes'}}) ;}
-      #if (ref $$tree{'/order'} eq 'ARRAY') { ++$n if @{$$tree{'/order'}} ;}
+      if (ref $$tree{'/nodes'} eq 'HASH')  { ++$n if (keys %{$$tree{'/nodes'}}) ;}
+      if (ref $$tree{'/order'} eq 'ARRAY') { ++$n if @{$$tree{'/order'}} ;}
 
       if ($ks > $n) { $addroot = 1 ;}
       else {
@@ -81,7 +74,7 @@ sub data {
   my ($data,$unicode) ;
   {
     my $parsed = {} ;
-    &_data(\$data , $tree , '' , -1 , {} , $parsed , undef , undef , $args{noident} , $args{nospace} , $args{lowtag} , $args{lowarg} , $args{wild} , $args{sortall} ) ;
+    &_data(\$data,$tree,'',-1, $parsed , $args{noident} , $args{nospace} , $args{lowtag} , $args{lowarg} , $args{wild} , $args{sortall} ) ;
     $data .= "\n" if !$args{nospace} ;
     if ( &_is_unicode($data) ) { $unicode = 1 ;}
   }
@@ -116,7 +109,7 @@ sub data {
     $meta = "\n" . join ("\n", @metas) ;
   }
   
-  my $wild = $args{wild} ? ' [format: wild]' : '' ;
+  my $wild = ' [format: wild]' if $args{wild} ;
   
   my $metagen = qq`\n<?meta name="GENERATOR" content="XML::Smart/$XML::Smart::VERSION$wild Perl/$] [$^O]" ?>` ;
   if ( $args{nometagen} ) { $metagen = '' ;}
@@ -130,15 +123,7 @@ sub data {
   
   if ( $args{noheader} ) { $xml = '' ; $metagen = '' if $args{nometagen} eq '' ;}
   
-  my $dtd ;
-  
-  if ( !$args{nodtd} && $$this->{DTD} ) {
-    $dtd = ref $$this->{DTD} ? $$this->{DTD}->CutDTD : $$this->{DTD} ;
-    $dtd =~ s/\s*$// ;
-    $dtd = "\n$dtd" if $dtd ne '' && !$args{nospace} ;
-  }
-  
-  $data = $xml . $metagen . $meta . $dtd . $data ;
+  $data = $xml . $metagen . $meta . $data ;
   
   if ($xml eq '') { $data =~ s/^\s+//gs ;}
   
@@ -178,20 +163,10 @@ sub is_valid_tree {
 ###############
 
 sub _is_unicode {
-  if ($] >= 5.008001) {
-    if ( utf8::is_utf8($_[0])) { return 1 ;}
-  }
-  elsif ($] >= 5.008) {
-    require Encode ;
-    if ( Encode::is_utf8($_[0])) { return 1 ;}
-  }
-  elsif ( $] >= 5.007 ) {
-    my $is = eval(q`
-      if ( $_[0] =~ /[\x{100}-\x{10FFFF}]/s) { return 1 ;}
-      return undef ;
+  if ($] >= 5.8) {
+    eval(q`
+      if ( $data =~ /[\x{100}-\x{10FFFF}]/s) { return 1 ;}}
     `);
-    $@ = undef ;
-    return 1 if $is ;
   }
   else {
     ## No Perl internal support for UTF-8! ;-/
@@ -207,20 +182,17 @@ sub _is_unicode {
 #########
 
 sub _data {
-  my ( $data , $tree , $tag , $level , $prev_tree , $parsed , $ar_i , $node_type , @stat ) = @_ ;
+  my ( $data , $tree , $tag , $level , $parsed , @stat ) = @_ ;
 
-  if (ref($tree) eq 'XML::Smart') { $tree = defined $$tree->{content} ? $$tree->{content} : $$tree->{point} ;}
+  if (ref($tree) eq 'XML::Smart') { $tree = $$tree->{point} ;}
   
-  if ( ref($tree) ) {
-    if ($$parsed{"$tree"}) { return ;}
-    ++$$parsed{"$tree"} ;
-  }
+  if ($$parsed{"$tree"}) { return ;}
+  $$parsed{"$tree"}++ ;
   
   my $ident = "\n" ;
   $ident .= '  ' x $level if !$stat[0] ;
-
+  
   if ($stat[1]) { $ident = '' ;}
-  $stat[1] -= 2 if $stat[1] > 1 ;
   
   my $tag_org = $tag ;
   $tag = $stat[4] ? $tag : &_check_tag($tag) ;
@@ -228,67 +200,38 @@ sub _data {
   elsif ($stat[2] == 2) { $tag = "\U$tag\E" ;}
 
   if (ref($tree) eq 'HASH') {
-    my ($args,$args_end,$tags,$cont,$stat_1) ;
+    my ($args,$args_end,$tags,$cont) ;
     
-    my (@all_keys , %multi_keys) ;
+    my @all_keys ;
     
     if ( !$stat[5] && $tree->{'/order'} ) {
       my %keys ;
-      foreach my $keys_i ( @{$tree->{'/order'}} ) {
-        if ( exists $$tree{$keys_i} && (!ref($$tree{$keys_i}) || ref($$tree{$keys_i}) eq 'HASH' || ref($$tree{$keys_i}) eq 'XML::Smart' || (ref($$tree{$keys_i}) eq 'ARRAY' && exists $$tree{$keys_i}[ $keys{$keys_i} ] ) ) ) {
+      foreach my $keys_i ( @{$tree->{'/order'}} , sort keys %$tree ) {
+        if ( !$keys{$keys_i} && exists $$tree{$keys_i} ) {
           push(@all_keys , $keys_i) ;
-          
-          if ( ++$keys{$keys_i} == 2 && ref $$tree{$keys_i} eq 'ARRAY' ) {
-            my @val = map { ( $_ ne '' ? 1 : () ) } @{ $$tree{$keys_i} } ;
-            $multi_keys{$keys_i} = 1 if $#val > 0 ;
-          }
+          $keys{$keys_i} = 1 ;
         }
       }
-      foreach my $keys_i ( sort keys %$tree ) {
-        if ( !$keys{$keys_i} && exists $$tree{$keys_i} ) { push(@all_keys , $keys_i) ;}
-      }
     }
     else { @all_keys = sort keys %$tree ;}
     
-    my %array_i ;
-
+    ##print "*** $tree->{'/order'} >> @all_keys\n" ;
+    
+    #if ( (defined $$tree{CONTENT} && $$tree{CONTENT} ne '') || (defined $$tree{content} && $$tree{content} ne '')) { $stat[0] = 1 ; $ident = '' ;}
+    
     foreach my $Key ( @all_keys ) {
       if ($Key eq '' || $Key eq '/order' || $Key eq '/nodes') { next ;}
-
-      if ( $Key eq '!--' && (!ref($$tree{$Key}) || ( ref($$tree{$Key}) eq 'HASH' && (keys %{$$tree{$Key}}) == 1 && (defined $$tree{$Key}{CONTENT} || defined $$tree{$Key}{content}) ) ) ) {
+      if (ref($$tree{$Key})) {
+        my $k = $$tree{$Key} ;
+        if (ref $k eq 'XML::Smart') { $k = ${$$tree{$Key}}->{point} ;}
+        $args .= &_data(\$tags,$k,$Key, $level+1 , $parsed , @stat) ;
+      }
+      elsif ("\L$Key\E" eq 'content') { $cont .= $$tree{$Key} ;}
+      elsif ( $Key eq '!--' && (!ref($$tree{$Key}) || ( ref($$tree{$Key}) eq 'HASH' && keys %{$$tree{$Key}} == 1 && (defined $$tree{$Key}{CONTENT} || defined $$tree{$Key}{content}) ) ) ) {
         my $ct = $$tree{$Key} ;
         if (ref $$tree{$Key}) { $ct = defined $$tree{$Key}{CONTENT} ? $$tree{$Key}{CONTENT} : $$tree{$Key}{content} ;} ;
-        if ( $ct ne '' ) { $tags .= "$ident<!-- $ct -->" ;}
+        $cont .= '<!--' . $ct . '-->' ;
       }
-      elsif (ref($$tree{$Key})) {
-        my $k = $$tree{$Key} ;
-        my $i ;
-        if (ref $k eq 'XML::Smart') {
-          $k = defined ${$$tree{$Key}}->{content} ? ${$$tree{$Key}}->{content} : ${$$tree{$Key}}->{point} ;
-        }
-        elsif ( ref $k eq 'ARRAY' && $multi_keys{$Key} ) {
-          $i = $array_i{$Key}++ if $#{$k} > 0 ;
-        }
-        $args .= &_data(\$tags,$k,$Key, $level+1 , $tree , $parsed , $i , $$tree{'/nodes'}{$Key} , @stat) if $array_i{$Key} ne 'ok' ;
-        $array_i{$Key} = 'ok' if $i eq '' && ref $k eq 'ARRAY' ;
-      }
-      elsif ( $$tree{'/nodes'}{$Key} ) {
-        my $k = [$$tree{$Key}] ;
-        $args .= &_data(\$tags,$k,$Key, $level+1 , $tree , $parsed , undef , $$tree{'/nodes'}{$Key} , @stat) ;
-      }
-      elsif (lc($Key) eq 'content') {
-        if ( tied($$tree{$Key}) && $$tree{$Key} =~ /\S/s ) {
-          $ident = '' ; $stat[1] += 2 ;
-        }
-        next if tied($$tree{$Key}) ;
-        
-        if ( $$tree{$Key} ne '' ) {
-          my $p0 = length($tags) ;
-          $tags .= $$tree{$Key} ;        
-          $cont = [$p0, length($tags) - $p0] ;
-        }
-      }
-      elsif ($Key =~ /^\/\.CONTENT\/\d+$/) { $tags .= $$tree{$Key} ;}
       elsif ( $stat[4] && $$tree{$Key} eq '') { $args_end .= " $Key" ;}
       else {
         my $tp = _data_type($$tree{$Key}) ;
@@ -318,65 +261,26 @@ sub _data {
       }
     }
     
-    foreach my $Key ( keys %array_i ) {
-      if ( $array_i{$Key} ne 'ok' && $#{ $$tree{$Key} } >= $array_i{$Key} ) {
-        for my $i ( $array_i{$Key} .. $#{ $$tree{$Key} } ) {
-          $args .= &_data(\$tags, $$tree{$Key} ,$Key, $level+1 , $tree , $parsed , $i , $$tree{'/nodes'}{$Key} , @stat) ;
-        }
-      }
-    }
-    
-    if ( $cont ne '' ) {
-      my ( $po , $p1 ) = @$cont ;
-      my $cont = substr($tags , $po , $p1) ;
-        
-      my $tp = _data_type($cont) ;
-      
-      if ( $node_type =~ /^(\w+),(\d+),(\d*)$/ ) {
-        my ( $node_tp , $node_set ) = ($1,$2) ;
-
-        if ( !$node_set ) {
-          if    ( $tp == 3 && $node_tp eq 'cdata'  ) { $tp = 0 ;}
-          elsif ( $tp == 4 && $node_tp eq 'binary' ) { $tp = 0 ;}
-        }
-        else {
-          if    ( $node_tp eq 'cdata'  ) { $tp = 3 ;}
-          elsif ( $node_tp eq 'binary' ) { $tp = 4 ;}
-        }
-      }
-      
-      if ( $tp == 3 ) { $cont = "<![CDATA[$cont]]>" ;}
-      elsif ( $tp == 4 ) {
-        require XML::Smart::Base64 ;
-        $cont = &XML::Smart::Base64::encode_base64($cont) ;
-        $cont =~ s/\s$//s ;
-        $args .= ' dt:dt="binary.base64"' ;
-      }
-      else { &_add_basic_entity($cont) ;}
-      
-      my $pe = $po + $p1 ;
-      my $px = $pe ;
-      while( substr($tags , $px , 1) =~ /\s/ ) { ++$px ;}
-
-      if ( $px > $pe ) { substr($tags , $pe , $px-$pe) = '' ;}
-      
-      substr($tags , $po , $p1) = $cont ;
-    }
-    
-    ##print "***$tag>> $args,$args_end,$tags,$cont,$stat_1 [@all_keys]\n" ;
+    &_add_basic_entity($cont) if $cont ne '' ;
     
     if ($args_end ne '') {
       $args .= $args_end ;
       $args_end = undef ;
     }
-
-    if (!@all_keys) {
-      $$data .= qq`$ident<$tag/>` if $tag ne '' ;
-    }
-    elsif ($args ne '' && $tags ne '') {
+    
+    #print "cont [$cont] $tags , $args , $cont\n" ;
+    
+    if ($args ne '' && $tags ne '') {
       $$data .= qq`$ident<$tag$args>` if $tag ne '' ;
       $$data .= $tags ;
-      $$data .= $ident if !$cont ;
+      $$data .= $cont ;
+      $$data .= $ident if $cont eq '' ;
+      $$data .= qq`</$tag>` if $tag ne '' ;
+    }
+    elsif ($args ne '' && $cont ne '') {
+      $$data .= qq`$ident<$tag$args>` if $tag ne '' ;
+      $$data .= $cont ;
+      $$data .= $ident if $cont eq '' ;
       $$data .= qq`</$tag>` if $tag ne '' ;
     }
     elsif ($args ne '') {
@@ -385,18 +289,19 @@ sub _data {
     elsif ($tags ne '') {
       $$data .= qq`$ident<$tag>` if $tag ne '' ;
       $$data .= $tags ;
-      $$data .= $ident if !$cont ;
+      $$data .= $cont ;
+      $$data .= $ident if $cont eq '' ;
       $$data .= qq`</$tag>` if $tag ne '' ;
     }
-    else {
-      $$data .= qq`$ident<$tag></$tag>` if $tag ne '' ;
+    elsif ($cont ne '') {
+      $$data .= qq`$ident<$tag>` if $tag ne '' ;
+      $$data .= $cont ;
+      $$data .= qq`</$tag>` if $tag ne '' ;
     }
   }
   elsif (ref($tree) eq 'ARRAY') {
     my ($c,$v,$tags) ;
-
-    foreach my $value_i ( ($ar_i ne '' ? $$tree[$ar_i] : @$tree) ) {
-      
+    foreach my $value_i (@$tree) {
       my $value = $value_i ;
       if (ref $value_i eq 'XML::Smart') { $value = $$value_i->{point} ;}
       
@@ -412,7 +317,7 @@ sub _data {
       elsif (ref($value)) {
         if (ref($value) eq 'HASH') {
           $c = 2 ;
-          &_data(\$tags,$value,$tag,$level, $tree , $parsed , undef , undef , @stat) ;
+          &_data(\$tags,$value,$tag,$level, $parsed , @stat) ;
           $do_val = 0 ;
         }
         elsif (ref($value) eq 'SCALAR') { $value = $$value ;}
@@ -420,19 +325,6 @@ sub _data {
       }
       if ( $do_val && $value ne '') {
         my $tp = _data_type($value) ;
-        
-        if ( $node_type =~ /^(\w+),(\d+),(\d*)$/ ) {
-          my ( $node_tp , $node_set ) = ($1,$2) ;
-          if ( !$node_set ) {
-            if    ( $tp == 3 && $node_tp eq 'cdata'  ) { $tp = 0 ;}
-            elsif ( $tp == 4 && $node_tp eq 'binary' ) { $tp = 0 ;}
-          }
-          else {
-            if    ( $node_tp eq 'cdata'  ) { $tp = 3 ;}
-            elsif ( $node_tp eq 'binary' ) { $tp = 4 ;}
-          }
-        }
-        
         if ($tp <= 2) {
           $c++ ;
           my $cont = $value ; &_add_basic_entity($value) ;
@@ -455,12 +347,11 @@ sub _data {
         }
       }
     }
-
-    if ( $ar_i eq '' && $c <= 1 && ! $$prev_tree{'/nodes'}{$tag}) {
+    if ($c <= 1) {
       my $k = $stat[4] ? $tag : &_check_key($tag) ;
       if    ($stat[3] == 1) { $k = "\L$k\E" ;}
       elsif ($stat[3] == 2) { $k = "\U$k\E" ;}
-      delete $$parsed{"$tree"} if ref($tree) ;
+      delete $$parsed{"$tree"} ;
       return " $k=" . &_add_quote($v) ;
     }
     else { $$data .= $tags ;}
@@ -469,25 +360,18 @@ sub _data {
     my $k = $stat[4] ? $tag : &_check_key($tag) ;
     if    ($stat[3] == 1) { $k = "\L$k\E" ;}
     elsif ($stat[3] == 2) { $k = "\U$k\E" ;}
-    delete $$parsed{"$tree"} if ref($tree) ;
+    delete $$parsed{"$tree"} ;
     return " $k=" . &_add_quote($$tree) ;
   }
   elsif (ref($tree)) {
     my $k = $stat[4] ? $tag : &_check_key($tag) ;
     if    ($stat[3] == 1) { $k = "\L$k\E" ;}
     elsif ($stat[3] == 2) { $k = "\U$k\E" ;}
-    delete $$parsed{"$tree"} if ref($tree) ;
+    delete $$parsed{"$tree"} ;
     return " $k=" . &_add_quote("$tree") ;
   }
-  else {
-    my $k = $stat[4] ? $tag : &_check_key($tag) ;
-    if    ($stat[3] == 1) { $k = "\L$k\E" ;}
-    elsif ($stat[3] == 2) { $k = "\U$k\E" ;}
-    delete $$parsed{"$tree"} if ref($tree) ;
-    return " $k=" . &_add_quote($tree) ;
-  }
 
-  delete $$parsed{"$tree"} if ref($tree) ;
+  delete $$parsed{"$tree"} ;
   return ;
 }
 
@@ -495,12 +379,12 @@ sub _data {
 # _DATA_TYPE #
 ##############
 
-## 4 binary
-## 3 CDATA
-## 2 content
-## 1 value
-
-sub _data_type { &XML::Smart::_data_type ;}
+sub _data_type {
+  return 4 if ($_[0] =~ /[^\w\d\s!"#\$\%&'\(\)\*\+,\-\.\/:;<=>\?\@\[\\\]\^\`\{\|}~]/s) ;
+  return 3 if ($_[0] =~ /<.*?>/s) ;
+  return 2 if ($_[0] =~ /[\r\n\t]/s) ;
+  return 1 ;
+}
 
 ##############
 # _CHECK_TAG #
@@ -522,6 +406,43 @@ sub _check_key {
   return( $_[0] ) ;
 }
 
+#######################
+# _PARSE_BASIC_ENTITY #
+#######################
+
+sub _parse_basic_entity {
+  $_[0] =~ s/&lt;/</gs ;
+  $_[0] =~ s/&gt;/>/gs ;
+  $_[0] =~ s/&amp;/&/gs ;
+  $_[0] =~ s/&apos;/'/gs ;
+  $_[0] =~ s/&quot;/"/gs ;
+  
+  $_[0] =~ s/&#(\d+);/ $1 > 255 ? pack("U",$1) : pack("C",$1)/egs ;
+  $_[0] =~ s/&#x([a-fA-F\d]+);/pack("U",hex($1))/egs ;
+  
+  return( $_[0] ) ;
+}
+
+#####################
+# _ADD_BASIC_ENTITY #
+#####################
+
+sub _add_basic_entity {
+  $_[0] =~ s/(&(?:\w+;)?)/{_is_amp($1) or $1}/sgex ;
+  $_[0] =~ s/</&lt;/gs ;
+  $_[0] =~ s/>/&gt;/gs ;
+  return( $_[0] ) ;
+}
+
+###########
+# _IS_AMP #
+###########
+
+sub _is_amp {
+  if ($_[0] eq '&') { return( '&amp;' ) ;}
+  return( undef ) ;
+}
+
 ##############
 # _ADD_QUOTE #
 ##############
@@ -532,8 +453,8 @@ sub _add_quote {
   
   &_add_basic_entity($data) ;
   
-  my $q1 = ($data =~ /"/s) ? 1 : undef ;
-  my $q2 = ($data =~ /'/s) ? 1 : undef ;
+  my $q1 = 1 if $data =~ /"/s ;
+  my $q2 = 1 if $data =~ /'/s ;
   
   if (!$q1 && !$q2) { return( qq`"$data"` ) ;}
   
