@@ -12,9 +12,6 @@
 
 package XML::Smart::Tie ;
 
-use strict ;
-no warnings ;
-
 ######################
 # _GENERATE_NULLTREE #
 ######################
@@ -22,34 +19,29 @@ no warnings ;
 sub _generate_nulltree {
   my $saver = shift ;
   my ( $K , $I ) = @_ ;
-
-  $saver->{null} = 0 ;
-
-  if ( !$saver->{keyprev} ) { return ;}
+  if ( !$saver->{keyprev} ) {
+    $saver->{null} = 0 ;
+    return ;
+  }
   
   my @tree = @{$saver->{keyprev}} ;
-  if (!@tree) { return ;}
+  if (!@tree) {
+    $saver->{null} = 0 ;
+    return ;  
+  }
   
   if ( $I > 0 ) { push(@tree , "[$I]") ;}
   
   my $tree = $saver->{tree} ;
   
-  my ($keyprev , $iprev , $treeprev , $array , $key , $i) ;
-  
-  ##print "GEN>> @tree\n" ;
-
+  my ($keyprev , $treeprev , $array , $key , $i) ;
   foreach my $tree_i ( @tree ) {
-    #print "*> $tree_i >> $keyprev # $iprev \n" ;
-    #use Data::Dumper ;
-    #print Dumper( [$tree , $treeprev , $array] ) ;
-    #print "=====================\n" ;
-    
     if (ref($tree) ne 'HASH' && ref($tree) ne 'ARRAY') {
       my $cont = $$treeprev{$keyprev} ;
       $$treeprev{$keyprev} = {} ;
       $$treeprev{$keyprev}{CONTENT} = $cont ;
     }
-
+      
     if ($tree_i =~ /^\[(\d+)\]$/) {
       $i = $1 ;
       if (exists $$treeprev{$keyprev}) {
@@ -66,16 +58,12 @@ sub _generate_nulltree {
       $tree = $$treeprev{$keyprev}[$i] ;
       $array = $$treeprev{$keyprev} ;
       $treeprev = $prev ;
-      $iprev = $i ;
     }
     elsif (ref $tree eq 'ARRAY') {
-      if (!exists $$tree[0] ) { $$tree[0] = {} ;}
-      if ( ref($$tree[0]) eq 'HASH' && !exists $$tree[0]{$tree_i} ) { $$tree[0]{$tree_i} = {} ;}
-
       my $prev = $tree ;
-      $tree = $$prev[0]{$tree_i} ;
-      $array = undef ;
-      $treeprev = $$prev[0] ;
+      $tree = $$treeprev{$keyprev}[0] ;
+      $array = $$treeprev{$keyprev} ;
+      $treeprev = $prev ;
     }
     else {
       if (exists $$tree{$tree_i}) {
@@ -88,33 +76,8 @@ sub _generate_nulltree {
           else { $$tree{$tree_i} = {} ;}
         }
       }
-      else {
-        if ( $treeprev ) {
-          if ( $array ) {
-            if ( ref($treeprev) eq 'ARRAY' ) {
-              push( @{ $$treeprev[$iprev]{'/order'} } , keys_valids($$treeprev[$iprev]) ) if !$$treeprev[$iprev]{'/order'} || !@{ $$treeprev[$iprev]{'/order'} } ;
-              push( @{ $$treeprev[$iprev]{'/order'} } , $tree_i) ;
-            }
-            else {
-              push( @{ $$treeprev{'/order'} } , keys_valids($treeprev) ) if !$$treeprev{'/order'} || !@{ $$treeprev{'/order'} } ;
-              push( @{ $$treeprev{'/order'} } , $tree_i ) ;
-            }
-          }
-          else {
-            if ( ref($treeprev) eq 'ARRAY' ) {
-              push( @{ $$treeprev[$iprev]{$keyprev}{'/order'} } , keys_valids($$treeprev[$iprev]{$keyprev}) ) if !$$treeprev[$iprev]{$keyprev}{'/order'} || !@{ $$treeprev[$iprev]{$keyprev}{'/order'} } ;
-              push( @{ $$treeprev[$iprev]{$keyprev}{'/order'} } , $tree_i) ;
-            }
-            else {
-              push( @{ $$treeprev{$keyprev}{'/order'} } , keys_valids($$treeprev{$keyprev}) ) if !$$treeprev{$keyprev}{'/order'} || !@{ $$treeprev{$keyprev}{'/order'} } ;
-              push( @{ $$treeprev{$keyprev}{'/order'} } , $tree_i ) ;
-            }
-          }
-        }
-        $$tree{$tree_i} = {} ;
-      }
+      else { $$tree{$tree_i} = {} ;}
       $keyprev = $tree_i ;
-      $iprev = undef ;
       $treeprev = $tree ;
       $tree = $$tree{$tree_i} ;
       $array = undef ;
@@ -122,6 +85,8 @@ sub _generate_nulltree {
     }
   }
   
+  #use Data::Dumper ; print Dumper( $saver->{tree} , $tree );
+  
   $saver->{point} = $tree ;
   $saver->{back} = $treeprev ;
   $saver->{array} = $array ;
@@ -129,25 +94,10 @@ sub _generate_nulltree {
   $saver->{i} = $i ;
 
   $saver->{null} = 0 ;
-  
-  ##use Data::Dumper ; print Dumper( $saver->{tree} , $saver->{point} , $saver->{back} , $saver->{array} );
 
   return( 1 ) ;
 }
 
-sub keys_valids {
-  my $tree = shift ;
-  return () if ref $tree ne 'HASH' ;
-  my @keys ;
-  
-  foreach my $Key (sort keys %$tree ) {
-    if ($Key eq '' || $Key eq '/order' || $Key eq '/nodes') { next ;}
-    push(@keys , $Key) ;
-  }
-  
-  return @keys ;
-}
-
 #################
 # _DELETE_XPATH #
 #################
@@ -174,14 +124,10 @@ sub FETCH {
   my $this = shift ;
   my ($i) = @_ ;
   my $key = $this->{saver}->{key} ;
-    
-  if ( $this->{saver}->{null} ) {
-    &XML::Smart::Tie::_generate_nulltree($this->{saver},$key,$i) ;
-  }
-
+  
   my $point = '' ;
-
-  #print "A-FETCH>> $key , $i >> @{$this->{saver}->{keyprev}} >> [$this->{saver}->{null}]\n" ;
+  
+  #print "A-FETCH>> $key , $i >> @{$this->{saver}->{keyprev}} [$this->{saver}->{null}]\n" ;
   
   if ($this->{saver}->{array}) {
     if (!exists $this->{saver}->{array}[$i] ) {
@@ -224,9 +170,6 @@ sub STORE {
   &XML::Smart::Tie::_delete_XPATH($this->{saver}) ;
   
   if ($this->{saver}->{array}) {
-    if ( !exists $this->{saver}->{array}[$i] && $key !~ /^\/\.CONTENT/ ) {
-      push( @{$this->{saver}->{back}->{'/order'}} , $key ) ;
-    }
     return $this->{saver}->{array}[$i] = $_[0] ;
   }
   elsif ($i == 0) {
@@ -238,20 +181,12 @@ sub STORE {
     }
   }
   else {
-    if ( exists $this->{saver}->{back}{$key}) {
+    if (exists $this->{saver}->{back}{$key}) {
       my $k = $this->{saver}->{back}{$key} ;
       $this->{saver}->{back}{$key} = [$k] ;
     }
     else { $this->{saver}->{back}{$key} = [] ;}
     $this->{saver}->{array} = $this->{saver}->{back}{$key} ;
-
-    if ( !exists $this->{saver}->{array}[$i] && $key !~ /^\/\.CONTENT/ ) {
-      if ( !exists $this->{saver}->{back}->{'/order'} ) {
-        my %keys = map { ( $_ eq '/order' || $_ eq '/nodes' ? () : ($_ => 1) ) } keys %{$this->{saver}->{back}} ;
-        push( @{$this->{saver}->{back}->{'/order'}} , sort keys %keys ) ;
-      }
-      push( @{$this->{saver}->{back}->{'/order'}} , $key ) ;
-    }
     return $this->{saver}->{array}[$i] = $_[0] ;
   }
 
@@ -302,7 +237,6 @@ sub DELETE {
   }
   elsif ($i == 0 && exists $this->{saver}->{back}{$key}) {
     my $k = $this->{saver}->{back}{$key} ;
-    delete $this->{saver}->{back}{'/nodes'}{$k} if defined $this->{saver}->{back}{'/nodes'} ;
     delete $this->{saver}->{back}{$key} ;
     return $k  ;
   }
@@ -330,11 +264,10 @@ sub PUSH {
   my $this = shift ;
   my $key = $this->{saver}->{key} ;
 
-  ##print "PUSH>> $key >> @{$this->{saver}->{keyprev}}\n" ;
+  #print "PUSH>> $key >> @{$this->{saver}->{keyprev}}\n" ;
 
-  my $gen_null ;
   if ( $this->{saver}->{null} ) {
-    $gen_null = &XML::Smart::Tie::_generate_nulltree($this->{saver},$key) ;
+    &XML::Smart::Tie::_generate_nulltree($this->{saver},$key,$i) ;
   }
   
   &XML::Smart::Tie::_delete_XPATH($this->{saver}) ;
@@ -343,7 +276,7 @@ sub PUSH {
     if (exists $this->{saver}->{back}{$key}) {
       if ( ref $this->{saver}->{back}{$key} ne 'ARRAY' ) {
         my $k = $this->{saver}->{back}{$key} ;
-        $this->{saver}->{back}{$key} = [ ( $gen_null ? () : $k) ] ;      
+        $this->{saver}->{back}{$key} = [$k] ;      
       }
     }
     else { $this->{saver}->{back}{$key} = [] ;}
@@ -358,9 +291,8 @@ sub UNSHIFT {
   my $this = shift ;
   my $key = $this->{saver}->{key} ;
 
-  my $gen_null ;
   if ( $this->{saver}->{null} ) {
-    $gen_null = &XML::Smart::Tie::_generate_nulltree($this->{saver},$key) ;
+    &XML::Smart::Tie::_generate_nulltree($this->{saver},$key,$i) ;
   }
   
   &XML::Smart::Tie::_delete_XPATH($this->{saver}) ;
@@ -369,7 +301,7 @@ sub UNSHIFT {
     if (exists $this->{saver}->{back}{$key}) {
       if ( ref $this->{saver}->{back}{$key} ne 'ARRAY' ) {
         my $k = $this->{saver}->{back}{$key} ;
-        $this->{saver}->{back}{$key} = [ ( $gen_null ? () : $k) ] ;      
+        $this->{saver}->{back}{$key} = [$k] ;      
       }
     }
     else { $this->{saver}->{back}{$key} = [] ;}
@@ -377,7 +309,7 @@ sub UNSHIFT {
     $this->{saver}->{point} = $this->{saver}->{back}{$key}[0] ;
   }
   
-  return unshift(@{$this->{saver}->{array}} , @_ ) ;
+  return unshift(@{$this->{saver}->{array}} , @_) ;
 }
 
 sub SPLICE {
@@ -388,7 +320,7 @@ sub SPLICE {
   my $key = $this->{saver}->{key} ;
   
   if ( $this->{saver}->{null} ) {
-    &XML::Smart::Tie::_generate_nulltree($this->{saver},$key) ;
+    &XML::Smart::Tie::_generate_nulltree($this->{saver},$key,$i) ;
   }
   
   &XML::Smart::Tie::_delete_XPATH($this->{saver}) ;
@@ -499,25 +431,13 @@ sub FETCH {
   my $this = shift ;
   my ( $key ) = @_ ;
   my $i ;
-  
-  if ( $this->{saver}->{null} ) {
-    &XML::Smart::Tie::_generate_nulltree($this->{saver},$key,$i) ;
-  }
 
-  #print "H-FETCH>> $key >> ". ( $this->{saver}->{keyprev} ? "@{$this->{saver}->{keyprev}}" : '' ) ."\n" ;
+  #print "H-FETCH>> $key , $i >> @{$this->{saver}->{keyprev}}\n" ;
 
-  #print "**FETCH>> $this->{saver}->{point}\n" ;
-  
   my $point = '' ;
   my $array ;
   
-  if (0&&ref($this->{saver}->{point}) eq 'ARRAY') {
-    $array = $this->{saver}->{point} ;
-    $point = $this->{saver}->{point}[0] ;
-    my $xml = &XML::Smart::clone($this->{saver},$point,undef,$array, undef,0) ;
-    return $xml->{$key} ;
-  }
-  elsif (ref($this->{saver}->{point}{$key}) eq 'ARRAY') {
+  if (ref($this->{saver}->{point}{$key}) eq 'ARRAY') {
     $array = $this->{saver}->{point}{$key} ;
     $point = $this->{saver}->{point}{$key}[0] ;
     $i = 0 ;
@@ -564,21 +484,15 @@ sub STORE {
   my $this = shift ;
   my $key = shift ;
 
-  ##print "H-STORE>> $key >> @{$this->{saver}->{keyprev}} >> [$this->{saver}->{null}]\n" ;
+  #print "H-STORE>> $key , $i >> @{$this->{saver}->{keyprev}} >> [$this->{saver}->{null}]\n" ;
   
   if ( $this->{saver}->{null} ) {
-    &XML::Smart::Tie::_generate_nulltree($this->{saver},$key) ;
+    &XML::Smart::Tie::_generate_nulltree($this->{saver},$key,$i) ;
   }
   
   &XML::Smart::Tie::_delete_XPATH($this->{saver}) ;
   
-  ##my @call = caller ;
-  ##print "***STORE>> $this->{saver}->{point} [@call] $_[0]\n" ;
-  
-  if (ref($this->{saver}->{point}) eq 'ARRAY') {
-    return $this->{saver}->{point}[0]{$key} = $_[0] ;
-  }
-  elsif ( ref($this->{saver}->{point}{$key}) eq 'ARRAY' ) {
+  if ( ref($this->{saver}->{point}{$key}) eq 'ARRAY' ) {
     return $this->{saver}->{point}{$key}[0] = $_[0] ;
   }
   else {
@@ -591,11 +505,9 @@ sub STORE {
     }
     
     if ( !exists $this->{saver}->{point}{$key} ) {
-      if ($key ne '/order' && $key ne '/nodes') {
-        if (!$this->{saver}->{keyorder}) { $this->_keyorder ;}
-        push(@{$this->{saver}->{keyorder}} , $key) ;
-        push(@{$this->{saver}->{point}{'/order'}} , $key ) ;
-      }
+      if (!$this->{saver}->{keyorder}) { $this->_keyorder ;}
+      push(@{$this->{saver}->{keyorder}} , $key) ;
+      push(@{$this->{saver}->{point}{'/order'}} , $key) ;
     }
     return $this->{saver}->{point}{$key} = $_[0] ;
   }
@@ -609,19 +521,6 @@ sub DELETE   {
   if ( exists $this->{saver}->{point}{$key} ) {
     &XML::Smart::Tie::_delete_XPATH($this->{saver}) ;
     $this->{saver}->{keyorder} = undef ;
-    
-    if ( defined $this->{saver}->{point}{'/order'} ) {
-      my (@order_ok , $set) ;
-      
-      foreach my $order_i ( @{ $this->{saver}->{point}{'/order'} } ) {
-        if ($order_i eq $key) { $set = 1 ;}
-        else { push(@order_ok , $order_i) ;}
-      }
-      
-      @{ $this->{saver}->{point}{'/order'} } = @order_ok if $set ;
-    }
-    
-    delete $this->{saver}->{point}{'/nodes'}{$key} if defined $this->{saver}->{point}{'/nodes'}{$key} ;
     return delete $this->{saver}->{point}{$key} ;
   }
   
@@ -669,96 +568,6 @@ sub _keyorder {
   $this->{saver}->{keyorder} = \@order ;
 }
 
-#########################
-# XML::SMART::TIESCALAR #
-#########################
-
-package XML::Smart::TieScalar ;
-
-sub TIESCALAR {
-  my $class = shift ;
-  my $this = bless( { p => $_[0] } , __PACKAGE__ ) ;
-  return $this ;
-}
-
-sub FETCH {
-  my $this = shift ;
-  my $wantarray = shift ;
-  
-  my ($data , @data) ;
-  foreach my $k_i ( $this->_get_content_keys ) {
-    if ( $wantarray ) { push(@data , $this->{p}{$k_i}) ;}
-    else { $data .= $this->{p}{$k_i} ;}
-  }
-  
-  return @data if $wantarray ;
-  return $data ;
-}
-
-sub STORE {
-  my $this = shift ;
-  my $i = $#_ > 0 ? shift : undef ;
-  
-  if ( $i =~ /^\d+$/ ) {
-    my $set ;
-    foreach my $k_i ( $this->_get_content_keys ) {
-      if ( $k_i =~ /^\/\.CONTENT\/$i$/ ) {
-        $this->{p}{$k_i} = $_[0] ;
-        $set = 1 ;
-        last ;
-      }
-    }
-    
-    if ( !$set ) {
-      $this->{p}{"/.CONTENT/$i"} = $_[0] ;
-      push( @{$this->{p}{'/order'}} , "/.CONTENT/$i") ;
-      $this->_cache_keys ;
-    }
-    
-    return $this->{p}{CONTENT} ;
-  }
-  
-  untie $this->{p}{CONTENT} ;
-
-  foreach my $k_i ( $this->_get_content_keys ) {
-    delete $this->{p}{$k_i} ;
-  }
-  
-  if ( $this->{p}{'/order'} ) {
-    my @order = @{$this->{p}{'/order'}} ;
-    my @order_ok ;
-    foreach my $order_i ( @order ) { push(@order_ok , $order_i) if $order_i !~ /^\/\.CONTENT\/\d+$/ ;}
-    if (@order_ok) { $this->{p}{'/order'} = \@order_ok ;}
-    else { delete $this->{p}{'/order'} ;}
-  }
-
-  $this->{p}{CONTENT} = $_[0] ;
-}
-
-sub UNTIE {}
-sub DESTROY {}
-
-sub _cache_keys {
-  my $this = shift ;
-  delete $this->{K} ;
-  my @keys = $this->_get_content_keys ;
-  $this->{K} = \@keys ;
-}
-
-sub _get_content_keys {
-  my $this = shift ;
-  return @{$this->{K}} if $this->{K} ;
-  
-  my %keys ;
-  foreach my $Key ( keys %{ $this->{p} } ) {
-    if ( $Key =~ /^\/\.CONTENT\/(\d+)$/ ) { $keys{$1} = $Key ;}
-  }
-  
-  my @keys = map { $keys{$_} } sort { $a <=> $b } keys %keys ;
-
-  return @keys ;
-}
-
 #######
 # END #
 #######
