@@ -23,9 +23,6 @@ our @EXPORT_OK = @EXPORT ;
 
 my $load_XPath ;
 
-use strict ;
-no warnings ;
-
 ##############
 # LOAD_XPATH #
 ##############
@@ -62,7 +59,6 @@ sub xpath {
   }
   
   if ( !@_ ) { return $xpath ;}
-  return ;
 }
 
 #################
@@ -79,7 +75,6 @@ sub xpath_pointer {
   my $xpath = XML::XPath->new(xml => $this->data_pointer(nospace => 1 , noheader => 1)) ;
   
   if ( !@_ ) { return $xpath ;}
-  return ;
 }
 
 #######
